/***************************************************************************
                          kgalaxyview.cpp  -  description
                             -------------------
    begin                : Wed Jul 10 2002
    copyright            : (C) 2002 by Marco Krohn
    email                : marco.krohn@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cmath>

#include <kapp.h>
#include <kdebug.h>

#include "kcanvas.h"
#include "kgalaxydoc.h"
#include "kgalaxyview.h"
#include "kmain.h"

#include "gamelogic/galaxy.h"
#include "common/point.h"
#include "common/rectangle.h"





//----------------------------------------  KGalaxyView  ----------------------------------------

KGalaxyView :: KGalaxyView( KGalaxyDoc* doc, QWidget* parent, const char* name, int wflags )
  : KCanvasView( doc, parent, name, wflags | WPaintClever | WNorthWestGravity | WRepaintNoErase )
{
  fDoc = doc;

  // widget receives mouse move events even if no buttons are pressed down
  viewport()->setMouseTracking( true );
  viewport()->setFocusPolicy( QWidget::ClickFocus );
//  setFocusPolicy( QWidget::ClickFocus );

  connect( this, SIGNAL(mousePosChanged(TPoint)), this, SLOT(showPos(TPoint)) );
  connect( this, SIGNAL(zoomChanged(double)), kapp->mainWidget(), SLOT(showZoom(double)) );

  setZoom( 1.0 );
}



KGalaxyView :: ~KGalaxyView()
{
}





//----------------------------------------  protected  ----------------------------------------

// virtual method
void KGalaxyView :: keyPressEvent( QKeyEvent* e ) 
{
  if ( e->text() == "+" )  {
    e->accept();
    fastZoomIn( getZoom()*1.1, 50, 50  );
  }
  else if ( e->text() == "-" )  {
    e->accept();
    fastZoomOut( getZoom()/1.1, 50, 50  );
  }
  else e->ignore();
}



// virtual method
void KGalaxyView :: wheelEvent( QWheelEvent* e )
{
  e->accept();                                             // accept the wheel event
  double newzoom = pow( 1.1, (double) e->delta() / 120.0 ) * getZoom();

  if ( e->delta() > 0 )  fastZoomIn( newzoom, e->x(), e->y() );
  else                   fastZoomOut( newzoom, e->x(), e->y() );
}



//----------------------------------------  protected slots  ----------------------------------------

void KGalaxyView :: showPos( TPoint pos )
{
  double w = fDoc->getGalaxy()->getSize().getX();
  if ( pos.getX() < 0 )  pos.setX( 0 );
  if ( pos.getX() > w )  pos.setX( w );

  double h = fDoc->getGalaxy()->getSize().getY();
  if ( pos.getY() < 0 )  pos.setY( 0 );
  if ( pos.getY() > h )  pos.setY( h );

  KMain* km = dynamic_cast<KMain*>( kapp->mainWidget() );
  km->showPos( pos );
}




/**
* $Log: kgalaxyview.cpp,v $
* Revision 1.4  2002/07/15 12:18:32  mkrohn5
* +/- for zooming
*
* Revision 1.3  2002/07/15 08:48:34  mkrohn5
* *** empty log message ***
*
* Revision 1.2  2002/07/15 08:08:57  mkrohn5
* various changes
*
* Revision 1.1  2002/07/12 09:53:25  mkrohn5
* initial import
*
*/
