/***************************************************************************
                          kmain.cpp  -  description
                             -------------------
    begin                : Fri Jul 12 2002
    copyright            : (C) 2002 by Marco Krohn
    email                : marco.krohn@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapp.h>
#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kstdaction.h>

#include "kgalaxydoc.h"
#include "kgalaxyview.h"
#include "kmain.h"

#include "admin/game-client.h"
#include "admin/game-create.h"
#include "admin/game-server.h"
#include "gamelogic/galaxy.h"
#include "gamelogic/universe.h"
#include "gamelogic/modules/system_basic.h"
#include "gamelogic/modules/fleet_basic.h"
#include "common/random.h"





const int KSTATUSBAR_ZOOM = 2;
const int KSTATUSBAR_POS  = 3;





//----------------------------------------  KMain  ----------------------------------------

// name=0
KMain :: KMain( const char* name ) : KMainWindow( 0, name )
{
  fDoc = 0;
  fClient = 0;

  createActions();
  createGUI();
  createStatusBar();
}



KMain :: ~KMain()
{
  reset();
}






//----------------------------------------  public slots  ----------------------------------------

void KMain :: showPos( TPoint p )
{
  QString msg = i18n("x: %1, y: %2").arg(p.getX(),0,'f',0).arg(p.getY(),0,'f',0);
  statusBar()->changeItem( msg, KSTATUSBAR_POS );
}



void KMain :: showZoom( double zoom )
{
  QString msg = i18n("zoom: %1").arg(zoom,0,'f',2);
  statusBar()->changeItem( msg, KSTATUSBAR_ZOOM );
}





//----------------------------------------  protected  ----------------------------------------

// virtual method
bool KMain :: queryClose()
{
  return slotClose();
}



bool KMain :: generateNewGame()
{
  bool ret = true;
  string gameDir = "../../stellarlegacy/stellarlegacy/testgame";

  cout << "Generating new game...." << endl;
  TGameCreate* create = new TGameCreate( gameDir, "no-one" );

  if ( create->createGame() == true )  {
    cout << "game successfully created!" << endl;
    cout << "saving server master file ..." << endl;
    ret &= create->saveServerMasterFile();
    delete create;                                         // cleaning up

    cout << "and now starting the game server ... " << endl;
    TGameServer* server = new TGameServer( gameDir, "no-one" );
    cout << "load the server master file ... " << endl;
    ret &= server->loadServerMasterFile();
    cout << "creating player data files ..." << endl;
    ret &= server->savePlayerDataFiles();
    delete server;
  }
  else  {
    cerr << "kmain:generateNewGame  FAILED!" << endl;  // fixme
    delete create;
    ret = false;
  }

  return ret;
}





//----------------------------------------  protected slots  ----------------------------------------

bool KMain :: slotClose()
{
  if ( fDoc == 0 )  return true;
  if ( fDoc->isModified() == false )  {
    reset();
    return true;
  }

  int ret = KMessageBox::warningYesNoCancel( this, i18n("Save modifications of this document?") );

  if ( ret == KMessageBox::Yes )  {                        // save changes first
    if ( slotSaveAs() == false )  return false;            // save doc--if saving fails, return false
    reset();
    return true;
  }
  else if ( ret == KMessageBox::No )  {                    // do not save changes
    reset();
    return true;
  }

  return false;
}



void KMain :: slotNew()
{                
  generateNewGame();
}


                
bool KMain :: slotOpen()
{
  string gameDir = "/home/antares/krohn/sl/testgame";
  string playerName = "Marco";

  kdDebug() << "creating game client ..." << endl;
  fClient = new TGameClient( gameDir, playerName );

  kdDebug() << "loading server master file ..." << endl;
  if ( fClient->loadServerMasterFile() == false )  {
    kdDebug() << "server master file could not be loaded! => loading failed" << endl;
    return false;
  }

  kdDebug() << "init player orders ..." << endl;
  if ( fClient->initializePlayerOrders(playerName) == false )  {
    kdDebug() << "failed!" << endl;
    return false;
  }

  // we simply use the first galaxy here (note that an universe can consist of more than one galaxy)
  TGalaxy* galaxy = *(TUniverse::instance()->getGalaxies()).begin();

  fDoc = new KGalaxyDoc( galaxy );
  KGalaxyView* view = new KGalaxyView( fDoc, this, 0, 0 );
  fDoc->addView( view );                                   // note: view is deleted by document
  setCentralWidget( view );
  view->show();
  view->setZoom( 1.0 );

  return true;
}



bool KMain :: slotSaveAs() const
{
  // fixme
  return false;
}





//----------------------------------------  private  ----------------------------------------

void KMain :: createActions()
{
  KStdAction::openNew( this, SLOT(slotNew()), actionCollection() );
  KStdAction::open( this, SLOT(slotOpen()), actionCollection() );
  KStdAction::close( this, SLOT(slotClose()), actionCollection() );
  KStdAction::quit( kapp, SLOT(closeAllWindows()), actionCollection() );
  KStdAction::save( this, SLOT(slotSave()), actionCollection() );
  KStdAction::saveAs( this, SLOT(slotSaveAs()), actionCollection() );
}



void KMain :: createStatusBar()
{
  statusBar()->insertFixedItem( "                      ", KSTATUSBAR_ZOOM, true );
  statusBar()->insertFixedItem( "                      ", KSTATUSBAR_POS, true );
  resetStatusBar();
}



void KMain :: reset()
{
  if ( fDoc != 0 )     delete fDoc;
  if ( fClient != 0 )  delete fClient;
  fDoc = 0;
  fClient = 0;

  resetStatusBar();
}



void KMain :: resetStatusBar()
{
  statusBar()->changeItem( "zoom:", KSTATUSBAR_ZOOM );
  statusBar()->changeItem( "x:   y:", KSTATUSBAR_POS );
}
