/***************************************************************************
                          kmain.h  -  description
                             -------------------
    begin                : Fri Jul 12 2002
    copyright            : (C) 2002 by Marco Krohn
    email                : marco.krohn@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __KMAIN_INCLUDED
#define __KMAIN_INCLUDED

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kmainwindow.h>

#include "common/point.h"

class KGalaxyDoc;
class KGalaxyView;
class TGalaxy;
class TGameClient;





//----------------------------------------  KMain  ----------------------------------------

/**
* The main window of the client.
*/
class KMain : public KMainWindow
{
  Q_OBJECT

  public:
    /**
    * Sets @a fDoc and @a fView to 0.
    */
    KMain( const char* name=0 );
    /**
    * Calls @ref reset.
    */
    ~KMain();

  public slots:
    /**
    * Shows the position of the mouse pointer with respect to the world
    * coordinates in the status bar of the main window.
    */
    void showPos( TPoint p );
    /**
    * Shows the zoom value in the status bar.
    * @param zoom The zoom value to show.
    */
    void showZoom( double zoom );

  protected:
    /**
    * Called before the window is closed, either by the user or indirectely by the
    * session manager.
    */
    virtual bool queryClose();
    // fixme
//    void createDemoGalaxy();
    bool generateNewGame();

  protected slots:
    void slotNew();
    bool slotClose();
    bool slotOpen();
    bool slotSaveAs() const;

  private:
    /**
    * The document for a galaxy. The document contains a pointer
    * to the real galaxy and also contains a list of views for this document.
    */
    KGalaxyDoc* fDoc;
    /**
    * The central object that stores all game related information.
    */
    TGameClient* fClient;

    /**
    * Creates all actions for the main window.
    */
    void createActions();
    /**
    * Creates the status bar for the main window.
    */
    void createStatusBar();
    /**
    * Resets all member variables.
    */
    void reset();
    void resetStatusBar();
};





#endif        // __KMAIN_INCLUDED
