// $Id: ra_fleet_engine_failure.cpp,v 1.9 2002/07/12 10:06:14 mkrohn5 Exp $

// Rule Atom FleetEngineFailure Code File
// Written by: Marco Krohn, <marco.krohn@gmx.de>

// Copyright (C) 2001 - 2002, Marco Krohn, <marco.krohn@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#include <iostream>

using namespace std;

#include "gamelogic/fleet.h"
#include "gamelogic/ruleatoms/ra_fleet_engine_failure.h"
#include "gamelogic/race.h"
#include "common/random.h"
#include "gamelogic/rulesystem.h"





//----------------------------------------  class TRAFleetEngineFailureBasic  ----------------------------------------

DefineRuleAtom(FleetEngineFailure,Basic,"fleet_engine_failure","basic");



// virtual method
bool RuleAtomMethod(FleetEngineFailure,Basic)::checkDependencies( list<TRuleSystem::tError>* ) const
{
  return true;
}



// virtual method
bool RuleAtomMethod(FleetEngineFailure,Basic)::execute()
{
  validate();
  fResults->set( "engine_failure", false );                // engines never fail:

  return true;
}





//----------------------------------------  class TRAFleetEngineFailureStars  ----------------------------------------

DefineRuleAtom(FleetEngineFailure,Stars,"fleet_engine_failure","stars");



// virtual method
bool RuleAtomMethod(FleetEngineFailure,Stars)::checkDependencies( list<TRuleSystem::tError>* ) const
{
  return true;
}



// virtual method
bool RuleAtomMethod(FleetEngineFailure,Stars)::execute()
{
  validate();

  fResults->set( "engine_failure", false );
  TFleet* fleet = dynamic_cast<TFleet*>( getInfoObject("fleet") );

  if ( fleet->getSpeed() >= 50.0 ) {
    TRace* creator = fleet->getCreator();
    if ( TRandom::getDouble() < creator->getFleetEngineFailure() ) {
      fResults->set( "engine_failure", true );
    }
  }

  return true;
}





/*
* $Log: ra_fleet_engine_failure.cpp,v $
* Revision 1.9  2002/07/12 10:06:14  mkrohn5
* include path fix
*
* Revision 1.8  2002/06/22 18:23:36  mkrohn5
* some changes caused by changes in ruleatom framework
*
* Revision 1.7  2002/06/21 08:31:56  mkrohn5
* changed method name in TDatapackageList:
* Get -> getAndDelete
*
* Revision 1.6  2002/06/07 00:34:05  mkrohn5
* name changes
*
* Revision 1.5  2002/06/05 00:32:35  mkrohn5
* various changes
*
* Revision 1.4  2002/06/04 09:28:01  mkrohn5
* some major changes: the ruleatoms now use macros and the prototype pattern
*
* Revision 1.3  2001/12/04 00:36:54  mkrohn5
* *** empty log message ***
*
* Revision 1.2  2001/12/04 00:32:03  mkrohn5
* a few corrections
*
* Revision 1.1  2001/12/03 01:37:48  mkrohn5
* - various updates and extensions
* - also two new rule atoms: item_access, fleet_engine_failure
*
*
*/
