// $Id: ra_fleet_engine_failure.h,v 1.7 2002/06/22 18:23:36 mkrohn5 Exp $

// "Rule Atom FleetEngineFailure" Header File
// Written by: Marco Krohn, <marco.krohn@gmx.de>

// Copyright (C) 2001 - , Marco Krohn, <marco.krohn@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#ifndef __RA_FLEET_ENGINE_FAILURE_INCLUDED
#define __RA_FLEET_ENGINE_FAILURE_INCLUDED

#include "gamelogic/ruleatom.h"





//----------------------------------------  class TRuleAtomFleetEngineFailure  ----------------------------------------

/**
* @class TRuleAtomFleetEngineFailure
* @brief The base class for fleet engine failure
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_fleet_engine_failure.h,v 1.7 2002/06/22 18:23:36 mkrohn5 Exp $
* @see TRuleAtom
*
* <dl compact><dt><strong>Needed Information:</strong><dd>
* "fleet" (TLinkable*) the fleet that wants to ignite the engines.<br>
* "wanted_speed" (@c double) the wanted velocity.</dl>
* <dl compact><dt><strong>Returns:</strong><dd>
* "engine_failure" (@c bool) contains the information if the fleet engine
* ignite or not.</dl>
*/
DeclareRuleAtomBase(FleetEngineFailure)
  CHECK("fleet");
  CHECK("wanted_speed");
EndDeclareRuleAtomBase;





//----------------------------------------  class TRAFleetEngineFailureBasic  ----------------------------------------

/**
* @class TRuleAtomFleetEngineFailureBasic
* @brief The basic rules for the rule "fleet_engine_failure".
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_fleet_engine_failure.h,v 1.7 2002/06/22 18:23:36 mkrohn5 Exp $
* @see TRuleAtomFleetEngineFailure
*
* <br><br><br>
* <dl compact><dt><strong>Name:</strong><dd>
* Fleet Engine Failure</dl>
* <dl compact><dt><strong>Ruleset:</strong><dd>
* Basic</dl>
* <dl compact><dt><strong>Description:</strong><dd>
* Fleet engines never fail.</dl>
* <dl compact><dt><strong>Needed modules:</strong><dd>
* none</dl>
*/
DeclareRuleAtom(FleetEngineFailure,Basic)
EndDeclareRuleAtom;





//----------------------------------------  class TRAFleetEngineFailureStars  ----------------------------------------

/**
* @class TRAFleetEngineFailureStars
* @brief The stars! rules for the rule "fleet_engine_failure".
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_fleet_engine_failure.h,v 1.7 2002/06/22 18:23:36 mkrohn5 Exp $
* @see TRuleAtomFleetEngineFailure
*
* <br><br><br>
* <dl compact><dt><strong>Name:</strong><dd>
* Fleet Engine Failure</dl>
* <dl compact><dt><strong>Ruleset:</strong><dd>
* Stars</dl>
* <dl compact><dt><strong>Description:</strong><dd>
* A fleet which moves >= 50 ly/turn (= warp7 in stars! terms)
* has a chance of TRace::getFleetEngineFailure() (refering to the creator race)
* that the engines will not ignite.</dl>
* <dl compact><dt><strong>Needed modules:</strong><dd>
* none</dl>
*/
DeclareRuleAtom(FleetEngineFailure,Stars)
EndDeclareRuleAtom;





#endif        // __RA_FLEET_ENGINE_FAILURE_INCLUDED
