// $Id: ra_fleet_movement.cpp,v 1.12 2002/07/12 10:06:14 mkrohn5 Exp $

// "Rule Atom Fleet Movement" Code File
// Written by: Marco Krohn, <marco.krohn@gmx.de>

// Copyright (C) 2001 - , Marco Krohn, <marco.krohn@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#include "gamelogic/fleet.h"
#include "gamelogic/galaxy.h"
#include "gamelogic/ruleatoms/ra_fleet_movement.h"

                                                            

                                                          

//----------------------------------------  class TRAFleetMovementBasic  ----------------------------------------

DefineRuleAtom(FleetMovement,Basic,"fleet_movement","basic");

                                                             

// virtual method
bool RuleAtomMethod(FleetMovement,Basic)::checkDependencies( list<TRuleSystem::tError>* ) const
{
  return true;
}



// virtual method
bool RuleAtomMethod(FleetMovement,Basic)::execute()
{
  validate();

  double time = getInfoDouble("time");
  TFleet* fleet = dynamic_cast<TFleet*>( getInfoObject("fleet") );
  TGalaxy* galaxy = fleet->getGalaxy();
  if ( fleet->hasWaypoint() == false )  return true;       // no waypoints => no movement

  // calculate the new position in space
  TPoint src = fleet->getPos();
  TPoint dest = fleet->getWaypointList()->getFirst()->getPos();       // where do we want to go to?
  TPoint dir = galaxy->getDistanceVector( src, dest );

  if ( dir.getLength() <= fleet->getSpeed()*time )  {
    src = dest;
  }
  else  {
    src += dir.normalize() * fleet->getSpeed()*time;
  }
  fleet->setPos( src );

  return true;
}





/*
* $Log: ra_fleet_movement.cpp,v $
* Revision 1.12  2002/07/12 10:06:14  mkrohn5
* include path fix
*
* Revision 1.11  2002/07/08 06:12:47  mkrohn5
* *** empty log message ***
*
* Revision 1.10  2002/06/22 18:23:36  mkrohn5
* some changes caused by changes in ruleatom framework
*
* Revision 1.9  2002/06/07 10:03:57  mkrohn5
* fixed the constant speed bug
*
* Revision 1.8  2002/06/07 00:34:30  mkrohn5
* improved fleet movement rule
*
* Revision 1.7  2002/06/05 00:32:35  mkrohn5
* various changes
*
* Revision 1.6  2002/06/04 09:28:01  mkrohn5
* some major changes: the ruleatoms now use macros and the prototype pattern
*
* Revision 1.5  2001/12/16 11:37:24  ishark
* TLink now uses dynamic_cast<>, lots of TAction-related fixes
*
* Revision 1.4  2001/12/05 00:40:47  mkrohn5
* - some conversions XXX -> xXX
* - implemented the basic rule "fleet movement"
*
* Revision 1.3  2001/12/04 00:32:03  mkrohn5
* a few corrections
*
* Revision 1.2  2001/12/03 01:37:48  mkrohn5
* - various updates and extensions
* - also two new rule atoms: item_access, fleet_engine_failure
*
* Revision 1.1  2001/11/25 22:11:57  mkrohn5
* created new subdir for rule atoms
*
*/

