// $Id: ra_item_access.h,v 1.1 2001/12/03 01:37:48 mkrohn5 Exp $

// "Rule Atom ItemAccess" Header File
// Written by: Marco Krohn, <marco.krohn@gmx.de>

// Copyright (C) 2001 - , Marco Krohn, <marco.krohn@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#ifndef __RA_ITEM_ACCESS_INCLUDED
#define __RA_ITEM_ACCESS_INCLUDED

#include "gamelogic/ruleatom.h"





//----------------------------------------  class TRAItemAccess  ----------------------------------------

/**
* @class TRAItemAccess
* @brief The base class for FIXME.
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_item_access.h,v 1.1 2001/12/03 01:37:48 mkrohn5 Exp $
* @see TRuleAtom
*
* This base class is mostly there for implementing the
* @ref #provideMissingInfo method. The following
* terms are used for the rule atom "item_access" (as usual a "*"
* signalizes that this informaion is needed):
*
* <ol>
*   <li>"item" [*] (TItem) = FIXME.</li>
* </ol>
*/
class TRAItemAccess : public TRuleAtom
{
  public:
    TRAItemAccess();

    virtual bool checkDependencies( const TRuleSystem* rules,
                                    list<TRuleSystem::tError>* errors ) const = 0;
    virtual bool execute() = 0;
    virtual bool provideMissingInfo();
};





//----------------------------------------  class TRAItemAccessBasic  ----------------------------------------

/**
* @class TRAItemAccessBasic
* @brief The basic rules for the rule "item_access".
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_item_access.h,v 1.1 2001/12/03 01:37:48 mkrohn5 Exp $
* @see TRuleAtom, TRAItemAccess
*
* This rule atom provides the basic rules for "item_access".
*/
class TRAItemAccessBasic : public TRAItemAccess
{
  public:
    TRAItemAccessBasic();

    virtual bool checkDependencies( const TRuleSystem* rules,
                                    list<TRuleSystem::tError>* errors ) const;
    virtual bool execute();
};





//----------------------------------------  class TRAItemAccessStars  ----------------------------------------

/**
* @class TRAItemAccessStars
* @brief The stars! rules for the rule "item_access".
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_item_access.h,v 1.1 2001/12/03 01:37:48 mkrohn5 Exp $
* @see TRuleAtom, TRAItemAccess
*
* This rule atom provides the stars! rules for "item_access".
*/
class TRAItemAccessStars : public TRAItemAccess
{
  public:
    TRAItemAccessStars();

    virtual bool checkDependencies( const TRuleSystem* rules,
                                    list<TRuleSystem::tError>* errors ) const;
    virtual bool execute();
};





//----------------------------------------  class TRAItemAccessSL  ----------------------------------------

/**
* @class TRAItemAccessSL
* @brief The Stellar Legacy rules for the rule "item_access".
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: ra_item_access.h,v 1.1 2001/12/03 01:37:48 mkrohn5 Exp $
* @see TRuleAtom, TRAItemAccess
*
* This rule atom provides the Stellar Legacy rules for "item_access".
*/
class TRAItemAccessSL : public TRAItemAccess
{
  public:
    TRAItemAccessSL();

    virtual bool checkDependencies( const TRuleSystem* rules,
                                    list<TRuleSystem::tError>* errors ) const;
    virtual bool execute();
};





#endif        // __RA_ITEM_ACCESS_INCLUDED
