#ifndef _CSTRING_H_
#define _CSTRING_H_

// ----------------------------------------------------------------------------
//
// Simple string class.
//
// Copyright (C) 1999, Pier Donini, Pier.Donini@epfl.ch
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details. 
//
// ----------------------------------------------------------------------------


//
// string
//

class string
{
private:
  char *fBuffer;

  void Allocate(const char *Chars);
  string& Append(const char *Chars);
  friend string Concat(const char *One, const char *Two);
  string& Set(const char *Chars);

public:

  // constructors
  string();
  string(const char *Chars);
  string(const string &String);

  // destructor
  ~string();

  // concatenation
  string& operator += (const char *Chars);
  string& operator += (const string &String);
  friend string operator + (const string &String, const char* Chars);
  friend string operator + (const char* Chars, const string &String);
  friend string operator + (const string &One, const string &Two);

  // assignment
  string& operator = (const char *Chars); 
  string& operator = (const string &String);

  // comparison
  friend bool operator == (const string &One, const string &Two);
  friend bool operator == (const char *Chars, const string &String);
  friend bool operator == (const string &String, const char *Chars);
  friend bool operator != (const string &One, const string &Two);
  friend bool operator != (const char *Chars, const string &String);
  friend bool operator != (const string &String, const char *Chars);

  // conversion
  const char* Chars() { return fBuffer; };
  operator const char*() { return fBuffer; };

  // misc
  char& operator[](unsigned int Index);
  unsigned int Length() { return strlen(fBuffer); };
};


//
// strings inlines
//

inline string::string(const char *Chars) 
{ 
  Allocate(Chars); 
}


inline string::string(const string &String) 
{ 
  Allocate(String.fBuffer); 
}


inline string& string::operator += (const char *Chars) 
{ 
  return Append(Chars); 
}


inline string& string::operator += (const string &String) 
{ 
  return Append(String.fBuffer); 
}


inline string operator + (const string &String, const char* Chars)
{
  return Concat(String.fBuffer, Chars);
}


inline string operator + (const char* Chars, const string &String)
{
  return Concat(Chars, String.fBuffer);
}


inline string operator + (const string &One, const string &Two)
{
  return Concat(One.fBuffer, Two.fBuffer);
}


inline string& string::operator = (const char *Chars)
{
  return Set(Chars);
}

inline string& string::operator = (const string &String)
{
  return Set(String.fBuffer);
}


inline bool operator == (const string &One, const string &Two)
{
  return strcmp(One.fBuffer, Two.fBuffer) == 0;
}


inline bool operator == (const char *Chars, const string &String)
{
  return strcmp(Chars, String.fBuffer) == 0;
}


inline bool operator == (const string &String, const char *Chars)
{
  return strcmp(String.fBuffer, Chars) == 0;
}


inline bool operator != (const string &One, const string &Two)
{
  return strcmp(One.fBuffer, Two.fBuffer) != 0;
}


inline bool operator != (const char *Chars, const string &String)
{
  return strcmp(Chars, String.fBuffer) != 0;
}


inline bool operator != (const string &String, const char *Chars)
{
  return strcmp(String.fBuffer, Chars) != 0;
}


#endif // _CSTRING_H_
