// ----------------------------------------------------------------------------
//
// Quick (and dirty) "cvs log" parser.
//
// Copyright (C) 1999, Pier Donini, Pier.Donini@epfl.ch
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details. 
//
//
// ----------------------------------------------------------------------------


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.h"
#include "cstring.h"

//
// Utils
//

struct tEntry
{
  string fAuthor;
  string fDate;
  string fHour;
  unsigned char fLines;
  long fStart;
};


int Greater(tEntry **One, tEntry **Two)
{
  int res = strcmp((*One)->fDate, (*Two)->fDate);

  if (res == 0)
    res = strcmp((*One)->fHour, (*Two)->fHour);

  if (res < 0)
    return -1;
  if (res > 0)
    return 1;

  return 0;
}


bool Starts(const char *arg1, const char *arg2)
{
  if (!*arg1)
    return false;

  for (; *arg1; arg1++, arg2++)
    if (*arg1 != *arg2)
      return false;

  return true;
}


//
// Main
//

int main(int argc, char *argv[])
{
  FILE *f;
  TList<tEntry *> entries;
  char line[160];

  if (argc != 2)
  {
    printf("Usage: %s <filename>\n", argv[0]);
    exit(0);
  }

  if ((f = fopen(argv[1], "r")) == NULL)
  {
    printf("No such file.\n");
    exit(0);
  }

  while (!feof(f) && fscanf(f, "%s", line))
    if (strcmp(line, "date:"))
      fgets(line, 160, f);	// read until eol
    else
    {
      /* date: 1994/08/31 12:52:57;  author: arda; */

      char date[11], hour[9], *tmp;
      string who;
      fscanf(f, "%s", date);
      date[10] = '\0';
      fscanf(f, "%s;", hour);
      hour[8] = '\0';
      fgets(line, 160, f);
      tmp = strtok(line, " ");
      tmp = strtok(NULL, ";");
      who = tmp;
      
      long start = ftell(f), position;
      int lines = 0;
      string comment;
      
      while (fgets(line, 160, f))
        if (Starts("----", line) || Starts("====", line))
          break;
        else
        {
          comment += line;
          lines++;
        }
      position = ftell(f);

      bool found = false;

      entries.Reset();
      while (!entries.AtTail())
      {
        tEntry *entry = entries.Next();
        if (entry->fDate == date && lines == entry->fLines)
        {
          string tmp;
          fseek(f, entry->fStart, SEEK_SET);
          for (int i = 0; i < entry->fLines; i++)
          {
            fgets(line, 160, f);
            tmp += line;
          }
          if (tmp == comment)
          {
            found = true;
            break;
          }
        }
      }

      if (!found)
      {
        tEntry *entry = new tEntry;
	entry->fAuthor = who;
        entry->fDate = date;
        entry->fHour = hour;
        entry->fStart = start;
        entry->fLines = lines;
        entries.AddLast(entry);
      }
      fseek(f, position, SEEK_SET);
    }

  entries.Sort(Greater);

  entries.ToTail();
  while (!entries.AtHead())
  {
    tEntry *entry = entries.Previous();
    printf("%-8s %s %s\n", entry->fAuthor.Chars(), entry->fDate.Chars(), entry->fHour.Chars());
    fseek(f, entry->fStart, SEEK_SET);
    for (int i = 0; i < entry->fLines; i++)
    {
      fgets(line, 160, f);
      printf("  %s", line);
    }
  }
  fclose(f);
  entries.Flush(DoFree);
}

