/***************************************************************************
                          log2html.cpp  -  description
                             -------------------
    begin                : Sat Apr 8 2000
    copyright            : (C) 2000 by Marco Krohn
    email                : mkrohn@stud.uni-hannover.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <fstream>
#include <iostream>
#include <string>

#include <stdio.h>

#include "log2html.h"

using namespace std;


int TLayer::iInfoCounter=0;

TErrorFile ErrorFile("error.html");





//----------------------------------------  TMessage  ----------------------------------------

// task: set values
TMessage :: TMessage()
{
  iIndent=0;
  iType=MESSAGE;
  sClassName="unknown";
  sFunctionName="unknown";
  sMessage="";
  bQouteMode=false;
  bReadStatus=false;
}



// returns: true=message read correctly, false=end of file reached (no message read)
bool TMessage :: ReadMessage(ifstream& in)
{
char Buffer[1024];
string sLine;
bool bMessageEnd;

  bQouteMode = false;
  bReadStatus = false;
  iType = TMessage::MESSAGE;

  while(!in.eof())
  {
    in.getline(Buffer, 1000);
    sLine=Buffer;
    bMessageEnd = ParseLine(sLine);
    if (bMessageEnd == true)
    {
      bReadStatus = true;
      return bReadStatus;                        // end of message found
    }
  }
  return bReadStatus;
}



//----------------------------------------  PRIVATE  ----------------------------------------

void TMessage :: ParseComment(string& sLine)
{
string sWord, lhs, rhs;
TStringTools ModStr;

  ModStr.ReadNextToken(sLine, true);             // read a ';'
  sWord = ModStr.ReadNextWord(sLine);            // read command name, e.g. class
  if (sWord == "indent")
  {
    ModStr.ReadEquation(sLine, lhs, rhs);
    iIndent=ModStr.ConvertToInteger(rhs);
  }
  else if (sWord == "type")
  {
    ModStr.ReadEquation(sLine, lhs, rhs);
    iType=ModStr.ConvertToInteger(rhs);
  }
  else if (sWord == "class")
  {
    ModStr.ReadNextToken(sLine, true);
    ModStr.ReadNextToken(sLine, true);
    ModStr.RemoveCharsFromStringBegin(sLine);
    ModStr.RemoveCharsFromStringEnd(sLine);
    sClassName=sLine;
  }
  else if (sWord == "function")
  {
    ModStr.ReadNextToken(sLine, true);
    ModStr.ReadNextToken(sLine, true);
    ModStr.RemoveCharsFromStringBegin(sLine);
    ModStr.RemoveCharsFromStringEnd(sLine);
    sFunctionName=sLine;
  }
}



// try to understand the meaning of the line, e.g. comment, command, output
// returns: true=end of message reached, false=otherwise
bool TMessage :: ParseLine(string& sLine)
{
  if (bQouteMode == true)
  {
    if (sLine == "\"")                      // end of qoute mode reached
    {
      TStringTools::RemoveCharsFromStringBegin(sMessage);
      TStringTools::RemoveCharsFromStringEnd(sMessage);
      TStringTools::Replace(sMessage, "\n", "<br>\n    ");
      return true;
    }
    else                                    // store line
    {
      sMessage += sLine +'\n';
      return false;
    }
  }
  else                                      // no quote found before
  {
    if (sLine.empty() == true)  return false;
    if (sLine == "\"")                      // begin of qoute mode
    {
      bQouteMode=true;
      return false;
    }
    else if (sLine[0] == ';')               // comment
    {
      ParseComment(sLine);
      return false;
    }
    else
    {
      cerr << "Parse Line: don't know what to do with line: " << endl
        << sLine << endl;
      exit(1);
    }
  }
  return false;
}





//----------------------------------------  TErrorFile  ----------------------------------------

TErrorFile :: TErrorFile(const string& sFileName)
{
  out.open(sFileName.c_str(), ios::out);
  BeginHTML();
}



TErrorFile :: ~TErrorFile()
{
  EndHTML();
  out.close();
}


void TErrorFile :: HandleMessage(TMessage* pMessage, int iInfoCounter)
{
  out << "<tr>" << endl
    << "  <td valign=\"top\"><a href=\"info-" << iInfoCounter << ".html\" target=\"info\">info</a>" << "</td>" << endl
    << "  <td>" << pMessage->GetMessage() << "</td>" << endl;

  if (pMessage->GetType() == TMessage::WARNING)  out << "  <td>WARNING</td>" << endl;
  if (pMessage->GetType() == TMessage::ERROR)  out << "  <td>ERROR</td>" << endl;
  out << "</tr>" << endl;
}



//----------------------------------------  PRIVATE  ----------------------------------------

void TErrorFile :: BeginHTML()
{
  out << "<html>" << endl
    << "<head>" << endl
    << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" << endl
    << "<title>Log File</title>" << endl
    << "</head>" << endl << endl
    << "<body text=\"#000000\" bgcolor=\"#FFFFFF\" link=\"#0000FF\" vlink=\"#000077\" alink=\"#FF0000\">" << endl
    << endl
    << "<table cellspacing=\"1\">" << endl
    << "<tr>" << endl
    << "  <th align=left width=\"30\">Info</th>" << endl
    << "  <th align=left width=\"80%\">Message</th>" << endl
    << "  <th align=left width=\"30\">Type</th>" << endl
    << "</tr>" << endl;
}



void TErrorFile :: EndHTML()
{
  out << endl
    << "</table>" << endl
    << "</body>" << endl
    << "</html>" << endl;
}





//----------------------------------------  TLayer  ----------------------------------------

// pParentLayer=0
TLayer :: TLayer(const string& sBaseFileName, int iMin, int iMax, const TLayer* pParentLayer)
{
string sFileName;

  this->pParentLayer = pParentLayer;
  pChildLayer = 0;
  pMessage = 0;
  iLayerCounter = 1;
  this->iMin = iMin;
  this->iMax = iMax;

  this->sBaseFileName = sBaseFileName;
  sFileName = sBaseFileName + ".html";
  out.open(sFileName.c_str(), ios::out);
  cout << sFileName.c_str() << endl;
  BeginHTML();
}



TLayer :: ~TLayer()
{
  if (pChildLayer != 0)  delete pChildLayer;     // clean up
  pChildLayer=0;
  EndHTML();                                     // write the end of the html file
  out.close();                                   // close the output file
}



// task: handles all messages with priority from iMin to iMax
// default: pNext=0
void TLayer :: HandleMessage(TMessage* pMessage, const TMessage* pNext=0)
{
  if (pMessage->GetIndentation()>=iMin && pMessage->GetIndentation()<=iMax)     // the message if for this layer
  {
    if (pChildLayer != 0)                        // close child layer
    {
      delete pChildLayer;
      pChildLayer=0;
    }
    if (pMessage != 0)  delete this->pMessage;   // delete the old message
    this->pMessage = pMessage;                   // save new message

    if (pNext == 0 || pNext->GetIndentation()<=iMax)  WriteMessage();
    else WriteMessage(MakeChildName()+".html");
  }
  else                                           // the message has a higher priority number
  {
    if (this->pMessage == 0)  CreateLinkMessage();    // in case the first line is a "more" link
    if (pChildLayer == 0)  CreateNewLayer();
    pChildLayer->HandleMessage(pMessage, pNext);
  }
}



//----------------------------------------  PRIVATE  ----------------------------------------

void TLayer :: BeginHTML()
{
  out << "<html>" << endl
    << "<head>" << endl
    << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" << endl
    << "<title>Log File</title>" << endl
    << "</head>" << endl << endl
    << "<body text=\"#000000\" bgcolor=\"#FFFFFF\" link=\"#0000FF\" vlink=\"#000077\" alink=\"#FF0000\">" << endl
    << "<p><a href=\"error.html\">Error and warnings</a></p>" << endl
   << endl;
  MakeLayerUpButton();
  WriteHistory();
  out << "<table cellspacing=\"1\">" << endl
    << "<tr>" << endl
    << "  <th align=left width=\"30\">Info</th>" << endl
    << "  <th align=left width=\"80%\">Message</th>" << endl
    << "  <th align=left width=\"30\">More</th>" << endl
    << "</tr>" << endl;
}



void TLayer :: CreateInfoFile()
{
ofstream o;
string sFileName;
char sNumber[10];

  sprintf(sNumber, "%i", iInfoCounter);
  sFileName="info-";  sFileName+=sNumber;  sFileName+=".html";
  o.open(sFileName.c_str(), ios::out);

  o << "<html>"
    << "<head>" << endl
    << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" << endl
    << "<title>Log File</title>" << endl
    << "</head>" << endl << endl
    << "<body text=\"#000000\" bgcolor=\"#FFFFFF\" link=\"#0000FF\" vlink=\"#000077\" alink=\"#FF0000\">" << endl
    << endl
    << "Class:    " << pMessage->GetClassName() << "<br>" << endl
    << "Function: " << pMessage->GetFunctionName() << "<br>" << endl
    << "</body>" << endl
    << "</html>";
  o.close();
}



void TLayer :: CreateLinkMessage()
{
  pMessage = new TMessage();
  pMessage->SetMessage("");
  pMessage->SetIndentation(iMax);
  out << "<tr>" << endl
    << "  <td valign=\"top\"></td>" << endl
    << "  <td></td>" << endl
    << "  <td valign=\"top\"><a href=\"" << MakeChildName() << ".html\">more</a></td>" << endl
    << "</tr>" << endl;
}



void TLayer :: CreateNewLayer()
{
string sNewBaseFileName;

  sNewBaseFileName = MakeChildName();
  pChildLayer = new TLayer(sNewBaseFileName, iMax+1, iMax+1, this);
  iLayerCounter++;
}



void TLayer :: EndHTML()
{
  out << endl
    << "</table>" << endl
    << "</body>" << endl
    << "</html>" << endl;
}



string TLayer :: MakeChildName()
{
char sNumber[10];
string sNewBaseFileName;

  sprintf(sNumber, "%i", iLayerCounter);
  sNewBaseFileName = sBaseFileName + "-" + sNumber;
  return sNewBaseFileName;
}



void TLayer :: MakeLayerUpButton()
{
  if (pParentLayer == 0)  return;
  out << "<a href=\"" << pParentLayer->sBaseFileName << ".html\">layer up</a>" << endl;
}



// task: writes the previous messages to the file
void TLayer :: WriteHistory()
{
int i;
const TLayer* p;

  p = this;
  while (p->pParentLayer != 0)  p=p->pParentLayer;

  i=1;
  while (p!=0)
  {
    if (p->pMessage != 0)
      out << "<h3>" << i << ". " << p->pMessage->GetMessage() << "</h3>" << endl;
    i++;
    p=p->pChildLayer;
  }
}



// default: sChildFileName=""
void TLayer :: WriteMessage(string sChildFileName)
{
  out << "<tr>" << endl
    << "  <td valign=\"top\"><a href=\"info-" << iInfoCounter << ".html\" target=\"info\">info</a>" << "</td>" << endl;

  if (pMessage->GetType() == TMessage::MESSAGE)
    out << "  <td>" << pMessage->GetMessage() << "</td>" << endl;
  if (pMessage->GetType()==TMessage::WARNING || pMessage->GetType()==TMessage::ERROR)
  {
    out << "  <td><strong>" << pMessage->GetMessage() << "</strong></td>" << endl;
    ErrorFile.HandleMessage(pMessage, iInfoCounter);
  }

  if (sChildFileName.empty() == true)            // without link
  {
    out << "  <td></td>" << endl;
  }
  else                                           // add a link to the child layer
  {
    out << "  <td valign=\"top\"><a href=\"" << sChildFileName << "\">more</a></td>" << endl;
  }
  out << "</tr>" << endl;

  CreateInfoFile();
  iInfoCounter++;
}










//----------------------------------------  TConvert  ----------------------------------------

TConvert :: TConvert()
{
  bEOF=false;
}



void TConvert::ConvertFile(const string& sFileName)
{
TLayer* pTopLayer;
list<TMessage*>::iterator first, next;
string sProjectName="sl";

  CreateMainHTML(sProjectName);
  in.open(sFileName.c_str(), ios::in);
  pTopLayer = new TLayer(sProjectName, 0, 0, 0);

  do
  {
    FillMessageList();
    next  = MessageList.begin();
    first = next++;

    if (MessageList.end() == next)  pTopLayer->HandleMessage(*first);
    else                            pTopLayer->HandleMessage(*first, *next);
    MessageList.erase(first);                    // remove element from list
  }
  while(MessageList.empty() == false);           // as long as there are messages in the queue

  delete pTopLayer;                              // close the top layer
  in.close();                                    // close the input file
}



//----------------------------------------  PRIVATE  ----------------------------------------

void TConvert :: CreateMainHTML(const string& sFileName)
{
ofstream o;

  o.open("index.html", ios::out);
  o << "<html>" << endl
    << "<head>" << endl
    << "<title>Log File Viewer - " << sFileName << "</title>" << endl
    << "</head>" << endl
    << endl
    << "<frameset rows=\"20%, 80%\">" << endl
    << "  <frame src=\"empty.html\" name=\"info\">" << endl
    << "  <frame src=\"" << sFileName << ".html\" name=\"data\">" << endl
    << "</frameset>" << endl
    << endl
    << "<noframes>" << endl
    << "<h2>Frame Alert</h2>" << endl
    << "<p>" << endl
    << "This document is designed to be viewed using the frames feature." << endl
    << "If you see this message, you are using a non-frame-capable web client.</p>" << endl
    << "</html>" << endl;
  o.close();
}



void TConvert :: FillMessageList()
{
int i;
bool bRead;
TMessage* pMessage;

  if (MessageList.size() >=5)  return;           // There are enough messages in the list.
  if (bEOF == true)  return;                     // End of file has been reached.

  for (i=0; i<10; i++)                           // Otherwise try to read 10 new messages.
  {
    pMessage = new TMessage;
    bRead = pMessage->ReadMessage(in);           // Load a message from input stream.
    if (bRead == false)                          // Reading the message failed.
    {
      bEOF=true;
      delete pMessage;
      return;
    }
    MessageList.push_back(pMessage);
  }
}