/***************************************************************************
                          stringtools.cpp  -  description
                             -------------------
    begin                : Sat Apr 8 2000
    copyright            : (C) 2000 by Marco Krohn
    email                : mkrohn@stud.uni-hannover.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdlib.h>

#include "stringtools.h"





TStringTools::TStringTools()
{
  sWordElements = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
  sSepElements = " \t\n\r";
  sStringStartSeq = "\"";
  sStringStopSeq = "\"";
  sStringStopIgnore = "\\\"";
}



// static function
int TStringTools::ConvertToInteger(const string& str)
{
  return atoi(str.data());
}



// static function
int TStringTools::CountNumberOfChars(const string& str, char c)
{
  int i, iLen, iCount=0;

  if (str.empty() == true)  return 0;
  iLen=str.size();
  i=0;
  while (i<iLen)
  {
    if (str[i] == c)  iCount++;
    i++;
  }
  return iCount;
}



// static function
void TStringTools::DeleteLastChar(string& str)
{
  if (str.empty() == true)  return;
  str.erase(str.size()-1, 1);
}



// bRemove=false
bool TStringTools::ReadEquation(string& str, string& lhs, string& rhs, bool bRemove)
{
  string temp=str;      // in case this isn't a well formed equation
  string seps;

  sSepElements.append("=");
  lhs=ReadNextToken(str, true);
  seps=ReadNextBlock(str, sSepElements, true);
  if (CountNumberOfChars(seps, '=') != 1)      // bad formed equation
  {
    lhs="";
    rhs="";
    str=temp;
    DeleteLastChar(sSepElements);
    return false;
  }
  else rhs=ReadNextToken(str, true);
  DeleteLastChar(sSepElements);

  if (bRemove == false)  str=temp;
  return true;
}




// bRemove=false
string TStringTools::ReadNextBlock(string& str, const string& sElements, bool bRemove) const
{
  int iPos;

  iPos = str.find_first_not_of(sElements, 0);
  if (iPos == (int) string::npos)  return "";

  if (bRemove == true && iPos > 0)   // remove some chars
  {
    string sWord = str.substr(0, iPos);
    str.erase(0, iPos);
    return sWord;
  }

  return str.substr(0, iPos);
}



// bRemove=false
string TStringTools::ReadNextToken(string& str, bool bRemove) const
{
  int iPos1, iPos2;

  iPos1 = str.find_first_not_of(sSepElements);
  if (iPos1 == (int) string::npos)  return "";
  iPos2 = str.find_first_of(sSepElements, iPos1);
  if (iPos2 == (int) string::npos)  iPos2=str.size();            // end of line reached

  if (bRemove == true && iPos2-iPos1 > 0)   // remove some chars
  {
    string sWord = str.substr(iPos1, iPos2-iPos1);
    str.erase(0, iPos2);
    return sWord;
  }

  return str.substr(iPos1, iPos2-iPos1);
}



// bRemove=false
string TStringTools::ReadNextWord(string& str, bool bRemove) const
{
  int iPos1, iPos2;

  iPos1 = str.find_first_not_of(sSepElements);
  if (iPos1 == (int) string::npos)  return "";
  iPos2 = str.find_first_not_of(sWordElements, iPos1);
  if (iPos2 == (int) string::npos)  return "";

  if (bRemove == true && iPos2-iPos1 > 0)   // remove some chars
  {
    string sWord = str.substr(iPos1, iPos2-iPos1);
    str.erase(0, iPos2);
    return sWord;
  }

  return str.substr(iPos1, iPos2-iPos1);
}



// static function
// sRemove=" \t\n\r"
void TStringTools::RemoveCharsFromStringBegin(string& str, const string sRemove)
{
int iPos;

  iPos = str.find_first_not_of(sRemove);
  if (iPos == (int) string::npos)  { str.erase(0, str.size());  return; }     // end of line reached
  str.erase(0, iPos);
}



// static function
// sRemove=" \t\n\r"
void TStringTools::RemoveCharsFromStringEnd(string& str, const string sRemove)
{
  int i, iLen;

  if ((iLen=str.size()) == 0)  return;
  for (i=iLen-1; i>=0; i--)
  {
    if (sRemove.find_first_of(str[i]) == string::npos)  break;       // character not found
  }
  if (i >= iLen-1)  return;  // nothing has changed
  str.erase(i+1, iLen-1-i);
}



// static function
void TStringTools::Replace(string& str, const string& search, const string replace)
{
  int iPos;
  string sTemp;

  if (search.empty() == true)  return;

  while( (iPos=str.find(search)) != (int) string::npos )
  {
    if (iPos > 0)  sTemp += str.substr(0, iPos);
    str.erase(0, iPos+search.size());
    sTemp += replace;
  }
  sTemp += str;
  str = sTemp;
}



